
#include <dbus/dbus-glib.h>
#include <stdio.h>
#include <stdlib.h>	
#include <string.h>
#include <getopt.h>

#include "lsdbus.h"

int
evaluate_args(int argc, char *argv[])
{
	struct option const long_options[] = {
		  {"help", no_argument, 0, ARG_HELP},
		  {"version", no_argument, 0, ARG_VERSION},
		  {NULL, 0, NULL, 0}
	};
	char *prgname;
	int c;

	if((prgname = strrchr(argv[0],'/')) == NULL)
		prgname = argv[0];
	else prgname++;		/* ignore first slash*/
	argv[0] = prgname;

	while ((c = getopt_long (argc, argv, ARG_ALL, long_options, (int *) 0)) != EOF) {
		switch (c) {
			case ARG_VERSION:
				printf("%s, version %s, (c) 2002-2005 Matthias Grimm\n", PACKAGE, VERSION);
				return 1;
			case ARG_HELP:
			default:
				printf("%s - lists clients at dbus message bus.\n", prgname);
				printf("Usage: %s [OPTION]\n", prgname);
				printf ("Options:\n"
					"   -%c, --help               display this help text and exit\n"
					"   -%c, --version            display version information and exit\n",
					ARG_HELP, ARG_VERSION);
				return 1;
		}
	}
	return 0;
}

void
listNames(DBusGConnection *bus)
{
	GError *error;
	DBusGProxy *proxy, *tmp;
	char **name_list, **ptr;
	GString *uniqueID;
	gboolean rc;
	
    /* Create a proxy object for the "bus driver" (name "org.freedesktop.DBus") */
	proxy = dbus_g_proxy_new_for_name (bus,
	                                   DBUS_SERVICE_DBUS,
									   DBUS_PATH_DBUS,
		                               DBUS_INTERFACE_DBUS);

	/* Call ListNames method, wait for reply */
	error = NULL;
	rc = dbus_g_proxy_call (proxy,
		 	                "ListNames",
					        &error,
					        G_TYPE_INVALID,
		                    G_TYPE_STRV,
					        &name_list,
					        G_TYPE_INVALID);
	
	if (rc) {
		uniqueID = g_string_new ("--");
		/* Print the results */
		for (ptr = name_list; *ptr; ptr++)
			/* only print readable names and supress bare bus ids */
			if (**ptr != ':') {
				/* The DBus services doesn't give us the unigue connection id
				 * so filter it out here. "--" would be printed instead.
				 */
				if (strcmp(*ptr, DBUS_SERVICE_DBUS) != 0) {
					error = NULL;
					tmp = dbus_g_proxy_new_for_name_owner (bus,
	            			                       *ptr,
												   "", /* no path */
		                             			   "", /* no interface */
												   &error);
					if (!tmp) {
						/* new proxy gave an error so we can't retrieve the unigue
						 * bus id. Print "??" to show that
						 */
						g_string_assign(uniqueID, "??");
						g_error_free (error);
					} else {
						g_string_assign(uniqueID, dbus_g_proxy_get_bus_name(tmp));
						g_object_unref (tmp);
					}
				}
				g_print ("  %-30s (%s)\n", *ptr, uniqueID->str);
			}
				
		g_strfreev (name_list);
		g_object_unref (proxy);
		g_string_free(uniqueID, TRUE);
	} else {
		/* Just do demonstrate remote exceptions versus regular GError */
		if (error->domain == DBUS_GERROR && error->code == DBUS_GERROR_REMOTE_EXCEPTION)
			g_printerr ("Caught remote method exception %s: %s", dbus_g_error_get_name (error), error->message);
		else
		    g_printerr ("Error: %s\n", error->message);
		g_error_free (error);
	}
}

int
main (int argc, char **argv)
{
	DBusGConnection *bus;
	GError *error;
			    
	if ((evaluate_args(argc, argv)) == 1)
		return 0;  /* help text printed, we are finish here */
	
	g_type_init ();

	error = NULL;
	g_print ("Session bus:\n");
	if ((bus = dbus_g_bus_get (DBUS_BUS_SESSION, &error))) {
		listNames(bus);		
	} else {
		g_print ("  No session bus available.\n");
		g_error_free (error);
	}
	g_print ("\n");
		
	error = NULL;
	g_print ("System bus:\n");
	if ((bus = dbus_g_bus_get (DBUS_BUS_SYSTEM, &error))) {
		listNames(bus);		
	} else {
		g_print ("  No system bus available.\n");
		g_error_free (error);
	}
	g_print ("\n");
	return 0;
}

